@@ -3,22 +3,32 @@ import numpy as np
 import pytest
 
 from stock_pandas import StockDataFrame
-
-simple_list = [2, 3, 4, 5, 6, 7]
+from .common import (
+    simple_list,
+    create_stock
+)
 
 
 @pytest.fixture
 def stock():
-    return StockDataFrame({
-        'open': simple_list,
-        'close': [x + 1 for x in simple_list],
-        'high': [x + 10 for x in simple_list],
-        'low': [x - 1 for x in simple_list],
-        'volume': [x * 100 for x in simple_list]
+    return create_stock()
+
+
+def test_astype(stock):
+    stock = stock.astype({
+        'open': 'float',
+        'close': 'float'
     })
 
+    assert isinstance(stock, StockDataFrame)
+
+    open0 = stock.iloc[0]['open']
+    assert isinstance(open0, float)
+
 
 def test_ma(stock):
+    stock.alias('Open', 'open')
+
     ma = stock['ma:2']
 
     stock = StockDataFrame(stock)
@@ -36,12 +46,15 @@ def test_ma(stock):
         volume=[800]
     ))
 
-    stock = stock.append(new)
+    stock = stock.append(new, ignore_index=True)
+    assert isinstance(stock, StockDataFrame)
+
     ma2 = stock.exec('ma:2')
 
     print(ma2)
 
     assert list(ma2[1:]) == [*list_ma0, 8.5]
+    assert stock['Open'][0] == 2
 
 
 def test_aliases(stock):
